@@ -1,7 +1,7 @@
 import logging
 
 from idpyoidc.client.exception import ConfigurationError
-from idpyoidc.client.oauth2 import provider_info_discovery
+from idpyoidc.client.oauth2 import server_metadata
 from idpyoidc.message import oidc
 from idpyoidc.message.oauth2 import ResponseMessage
 
@@ -56,18 +56,21 @@ def add_redirect_uris(request_args, service=None, **kwargs):
             _uris = [v for k, v in _cbs.items() if not k.startswith("__")]
             request_args["redirect_uris"] = _uris
         else:
-            request_args["redirect_uris"] = _context.redirect_uris
+            request_args["redirect_uris"] = _context.metadata["redirect_uris"]
 
     return request_args, {}
 
 
-class ProviderInfoDiscovery(provider_info_discovery.ProviderInfoDiscovery):
+class ProviderInfoDiscovery(server_metadata.ServerMetadata):
     msg_type = oidc.Message
     response_cls = oidc.ProviderConfigurationResponse
     error_msg = ResponseMessage
+    service_name = "provider_info"
+
+    metadata_attributes = {}
 
     def __init__(self, client_get, conf=None):
-        provider_info_discovery.ProviderInfoDiscovery.__init__(self, client_get, conf=conf)
+        server_metadata.ServerMetadata.__init__(self, client_get, conf=conf)
 
     def update_service_context(self, resp, **kwargs):
         _context = self.client_get("service_context")
@@ -91,17 +94,25 @@ class ProviderInfoDiscovery(provider_info_discovery.ProviderInfoDiscovery):
         :param issuer: The issuer identifier
         """
         _context = self.client_get("service_context")
+        _entity = self.client_get("entity")
+
         if not pcr:
             pcr = _context.provider_info
 
         regreq = oidc.RegistrationRequest
 
-        _behaviour = _context.behaviour
+        _behaviour = _context.specs.behaviour
 
         for _pref, _prov in PREFERENCE2PROVIDER.items():
-            try:
-                vals = _context.client_preferences[_pref]
-            except KeyError:
+            if _pref in ["scope"]:
+                vals = _entity.get_usage_value(_pref)
+            else:
+                try:
+                    vals = _entity.get_metadata_value(_pref)
+                except KeyError:
+                    continue
+
+            if not vals:
                 continue
 
             try:
@@ -142,9 +153,11 @@ class ProviderInfoDiscovery(provider_info_discovery.ProviderInfoDiscovery):
             if _pref not in _behaviour:
                 raise ConfigurationError("OP couldn't match preference:%s" % _pref, pcr)
 
-        for key, val in _context.client_preferences.items():
+        for key, val in _entity.collect_metadata().items():
             if key in _behaviour:
                 continue
+            if key in ["jwks", "jwks_uri"]:
+                continue
 
             try:
                 vtyp = regreq.c_param[key]
@@ -157,5 +170,5 @@ class ProviderInfoDiscovery(provider_info_discovery.ProviderInfoDiscovery):
             if key not in PREFERENCE2PROVIDER:
                 _behaviour[key] = val
 
-        _context.behaviour = _behaviour
+        _context.specs.behaviour = _behaviour
         logger.debug("service_context behaviour: {}".format(_behaviour))
