@@ -8,7 +8,8 @@ class RefreshAccessToken(refresh_access_token.RefreshAccessToken):
     error_msg = oidc.ResponseMessage
 
     def get_authn_method(self):
+        _specs = self.client_get("service_context").specs
         try:
-            return self.client_get("service_context").behaviour["token_endpoint_auth_method"]
+            return _specs.behaviour["token_endpoint_auth_method"]
         except KeyError:
             return self.default_authn_method
