@@ -1,6 +1,6 @@
 MAXIMUM_CLOCK_SKEW = 300
 
-URL_ENCODED = 'application/x-www-form-urlencoded'
+URL_ENCODED = "application/x-www-form-urlencoded"
 JSON_ENCODED = "application/json"
 JOSE_ENCODED = "application/jose"
 
