@@ -7,6 +7,7 @@ from typing import List
 from typing import Optional
 
 from idpyoidc.configure import Base
+from idpyoidc.server.client_configure import verify_oidc_client_information
 from idpyoidc.server.scopes import SCOPE2CLAIMS
 
 logger = logging.getLogger(__name__)
@@ -205,8 +206,11 @@ class EntityConfiguration(Base):
             elif not _val:
                 logger.warning(f"{key} not configured, using default configuration values")
 
-            if key == "template_dir":
+            if key == "oidc_clients":
+                _val = verify_oidc_client_information(_val)
+            elif key == "template_dir":
                 _val = os.path.abspath(_val)
+
             if key == "keys":
                 if not self.key_conf:
                     setattr(self, "key_conf", _val)
@@ -223,6 +227,7 @@ class OPConfiguration(EntityConfiguration):
             "id_token": None,
             "login_hint2acrs": {},
             "login_hint_lookup": None,
+            "oidc_clients": {},
             "sub_func": {},
             "scopes_to_claims": {},
         }
@@ -447,9 +452,9 @@ DEFAULT_EXTENDED_CONF = {
                 "post_logout_uri_path": "post_logout",
                 "signing_alg": "ES256",
                 "frontchannel_logout_supported": True,
-                "frontchannel_logout_session_supported": True,
+                "frontchannel_logout_session_required": True,
                 "backchannel_logout_supported": True,
-                "backchannel_logout_session_supported": True,
+                "backchannel_logout_session_required": True,
                 "check_session_iframe": "check_session_iframe",
             },
         },
