@@ -7,7 +7,6 @@ from urllib.parse import unquote
 from urllib.parse import urlencode
 from urllib.parse import urlparse
 
-from cryptojwt import BadSyntax
 from cryptojwt import as_unicode
 from cryptojwt import b64d
 from cryptojwt.jwe.exception import JWEException
@@ -15,6 +14,7 @@ from cryptojwt.jws.exception import NoSuitableSigningKeys
 from cryptojwt.utils import as_bytes
 from cryptojwt.utils import b64e
 
+from idpyoidc.exception import ImproperlyConfigured
 from idpyoidc.exception import ParameterError
 from idpyoidc.exception import URIError
 from idpyoidc.message import Message
@@ -40,6 +40,7 @@ from idpyoidc.server.user_authn.authn_context import pick_auth
 from idpyoidc.time_util import utc_time_sans_frac
 from idpyoidc.util import rndstr
 from idpyoidc.util import split_uri
+from idpyoidc.util import importer
 
 logger = logging.getLogger(__name__)
 
@@ -90,10 +91,10 @@ def max_age(request):
 
 
 def verify_uri(
-    endpoint_context: EndpointContext,
-    request: Union[dict, Message],
-    uri_type: str,
-    client_id: Optional[str] = None,
+        endpoint_context: EndpointContext,
+        request: Union[dict, Message],
+        uri_type: str,
+        client_id: Optional[str] = None,
 ):
     """
     A redirect URI
@@ -223,10 +224,10 @@ def get_uri(endpoint_context, request, uri_type):
 
 
 def authn_args_gather(
-    request: Union[AuthorizationRequest, dict],
-    authn_class_ref: str,
-    cinfo: dict,
-    **kwargs,
+        request: Union[AuthorizationRequest, dict],
+        authn_class_ref: str,
+        cinfo: dict,
+        **kwargs,
 ):
     """
     Gather information to be used by the authentication method
@@ -278,6 +279,53 @@ def check_unknown_scopes_policy(request_info, client_id, endpoint_context):
         raise UnAuthorizedClientScope()
 
 
+def validate_resource_indicators_policy(request, context, **kwargs):
+    if "resource" not in request:
+        return oauth2.AuthorizationErrorResponse(
+            error="invalid_target",
+            error_description="Missing resource parameter",
+        )
+
+    resource_servers_per_client = kwargs["resource_servers_per_client"]
+    client_id = request["client_id"]
+
+    if isinstance(resource_servers_per_client, dict) and client_id not in resource_servers_per_client:
+        return oauth2.AuthorizationErrorResponse(
+            error="invalid_target",
+            error_description=f"Resources for client {client_id} not found",
+        )
+
+    if isinstance(resource_servers_per_client, dict):
+        permitted_resources = [res for res in resource_servers_per_client[client_id]]
+    else:
+        permitted_resources = [res for res in resource_servers_per_client]
+
+    common_resources = list(set(request["resource"]).intersection(set(permitted_resources)))
+    if not common_resources:
+        return oauth2.AuthorizationErrorResponse(
+            error="invalid_target",
+            error_description=f"Invalid resource requested by client {client_id}",
+        )
+
+    common_resources = [r for r in common_resources if r in context.cdb.keys()]
+    if not common_resources:
+        return oauth2.AuthorizationErrorResponse(
+            error="invalid_target",
+            error_description=f"Invalid resource requested by client {client_id}",
+        )
+
+    if client_id not in common_resources:
+        common_resources.append(client_id)
+
+    request["resource"] = common_resources
+
+    permitted_scopes = [context.cdb[r]["allowed_scopes"] for r in common_resources]
+    permitted_scopes = [r for res in permitted_scopes for r in res]
+    scopes = list(set(request.get("scope", [])).intersection(set(permitted_scopes)))
+    request["scope"] = scopes
+    return request
+
+
 class Authorization(Endpoint):
     request_cls = oauth2.AuthorizationRequest
     response_cls = oauth2.AuthorizationResponse
@@ -305,6 +353,8 @@ class Authorization(Endpoint):
 
     def __init__(self, server_get, **kwargs):
         Endpoint.__init__(self, server_get, **kwargs)
+
+        self.resource_indicators_config = kwargs.get("resource_indicators", None)
         self.post_parse_request.append(self._do_request_uri)
         self.post_parse_request.append(self._post_parse_request)
         self.allowed_request_algorithms = AllowedAlgorithms(ALG_PARAMS)
@@ -462,8 +512,45 @@ class Authorization(Endpoint):
         else:
             request["redirect_uri"] = redirect_uri
 
+        if ("resource_indicators" in _cinfo
+            and "authorization_code" in _cinfo["resource_indicators"]):
+            resource_indicators_config = _cinfo["resource_indicators"]["authorization_code"]
+        else:
+            resource_indicators_config = self.resource_indicators_config
+
+        if resource_indicators_config is not None:
+            if "policy" not in resource_indicators_config:
+                policy = {"policy": {"callable": validate_resource_indicators_policy}}
+                resource_indicators_config.update(policy)
+            request = self._enforce_resource_indicators_policy(request, resource_indicators_config)
+
         return request
 
+    def _enforce_resource_indicators_policy(self, request, config):
+        _context = self.server_get("endpoint_context")
+
+        policy = config["policy"]
+        callable = policy["callable"]
+        kwargs = policy.get("kwargs", {})
+
+        if kwargs.get("resource_servers_per_client", None) is None:
+            kwargs["resource_servers_per_client"] = {
+                request["client_id"]: request["client_id"]
+            }
+
+        if isinstance(callable, str):
+            try:
+                fn = importer(callable)
+            except Exception:
+                raise ImproperlyConfigured(f"Error importing {callable} policy callable")
+        else:
+            fn = callable
+        try:
+            return fn(request, context=_context, **kwargs)
+        except Exception as e:
+            logger.error(f"Error while executing the {fn} policy callable: {e}")
+            return self.error_cls(error="server_error", error_description="Internal server error")
+
     def pick_authn_method(self, request, redirect_uri, acr=None, **kwargs):
         _context = self.server_get("endpoint_context")
         auth_id = kwargs.get("auth_method_id")
@@ -522,24 +609,33 @@ class Authorization(Endpoint):
         return _res
 
     def _unwrap_identity(self, identity):
+        # identity is a dict or a json object
+        # the value of 'uid' in the dictionary might be a base64 encoded (b64e) json object
         if isinstance(identity, dict):
+            _uid = as_unicode(identity['uid'])
             try:
-                _id = b64d(as_bytes(identity["uid"]))
-            except BadSyntax:
+                _id = b64d(as_bytes(_uid))
+            except Exception as err:
                 return identity
         else:
-            _id = b64d(as_bytes(identity))
+            try:
+                _id = b64d(as_bytes(identity))
+            except Exception as err:
+                return identity
 
-        return json.loads(as_unicode(_id))
+        try:
+            return json.loads(as_unicode(_id))
+        except UnicodeDecodeError:
+            return identity
 
     def setup_auth(
-        self,
-        request: Optional[Union[Message, dict]],
-        redirect_uri: str,
-        cinfo: dict,
-        cookie: List[dict] = None,
-        acr: str = None,
-        **kwargs,
+            self,
+            request: Optional[Union[Message, dict]],
+            redirect_uri: str,
+            cinfo: dict,
+            cookie: List[dict] = None,
+            acr: str = None,
+            **kwargs,
     ) -> dict:
         """
 
@@ -595,21 +691,22 @@ class Authorization(Endpoint):
                         if _csi.is_active() is False:
                             identity = None
 
-        authn_args = authn_args_gather(request, authn_class_ref, cinfo, **kwargs)
         _mngr = _context.session_manager
         _session_id = ""
 
         # To authenticate or Not
         if not identity:  # No!
             logger.info("No active authentication")
-            logger.debug("Known clients: {}".format(list(_context.cdb.keys())))
+            # logger.debug("Known clients: {}".format(list(_context.cdb.keys())))
 
             if "prompt" in request and "none" in request["prompt"]:
                 # Need to authenticate but not allowed
                 return self._login_required_error(redirect_uri, request)
             else:
+                authn_args = authn_args_gather(request, authn_class_ref, cinfo, **kwargs)
                 return {"function": authn, "args": authn_args}
         else:
+            authn_args = authn_args_gather(request, authn_class_ref, cinfo, **kwargs)
             logger.info(f"Active authentication: {identity}")
             if re_authenticate(request, authn):
                 # demand re-authentication
@@ -662,12 +759,12 @@ class Authorization(Endpoint):
         return ""
 
     def response_mode(
-        self,
-        request: Union[dict, AuthorizationRequest],
-        response_args: Optional[Union[dict, AuthorizationResponse]] = None,
-        return_uri: Optional[str] = "",
-        fragment_enc: Optional[bool] = None,
-        **kwargs,
+            self,
+            request: Union[dict, AuthorizationRequest],
+            response_args: Optional[Union[dict, AuthorizationResponse]] = None,
+            return_uri: Optional[str] = "",
+            fragment_enc: Optional[bool] = None,
+            **kwargs,
     ) -> dict:
         resp_mode = request["response_mode"]
         if resp_mode == "form_post":
@@ -741,10 +838,17 @@ class Authorization(Endpoint):
             _mngr = _context.session_manager
             _sinfo = _mngr.get_session_info(sid, grant=True)
 
+            scope = []
+            resource_scopes = []
             if request.get("scope"):
-                aresp["scope"] = _context.scopes_handler.filter_scopes(
-                    request["scope"], _sinfo["client_id"]
-                )
+                scope = request.get("scope")
+            if request.get("resource"):
+                resource_scopes = [_context.cdb[s]["scope"] for s in request.get("resource") if s in _context.cdb.keys() and _context.cdb[s].get("scope")]
+                resource_scopes = [item for sublist in resource_scopes for item in sublist]
+
+            aresp["scope"] = _context.scopes_handler.filter_scopes(
+                list(set(scope+resource_scopes)), _sinfo["client_id"]
+            )
 
             rtype = set(request["response_type"][:])
             handled_response_type = []
@@ -759,7 +863,7 @@ class Authorization(Endpoint):
                 _code = self.mint_token(
                     token_class="authorization_code",
                     grant=grant,
-                    session_id=_sinfo["session_id"],
+                    session_id=_sinfo["branch_id"],
                 )
                 aresp["code"] = _code.value
                 handled_response_type.append("code")
@@ -770,7 +874,7 @@ class Authorization(Endpoint):
                 _access_token = self.mint_token(
                     token_class="access_token",
                     grant=grant,
-                    session_id=_sinfo["session_id"],
+                    session_id=_sinfo["branch_id"],
                 )
                 aresp["access_token"] = _access_token.value
                 aresp["token_type"] = "Bearer"
@@ -796,7 +900,7 @@ class Authorization(Endpoint):
                     id_token = self.mint_token(
                         token_class="id_token",
                         grant=grant,
-                        session_id=_sinfo["session_id"],
+                        session_id=_sinfo["branch_id"],
                         scope=request["scope"],
                         **kwargs,
                     )
@@ -966,10 +1070,10 @@ class Authorization(Endpoint):
         return kwargs
 
     def process_request(
-        self,
-        request: Optional[Union[Message, dict]] = None,
-        http_info: Optional[dict] = None,
-        **kwargs,
+            self,
+            request: Optional[Union[Message, dict]] = None,
+            http_info: Optional[dict] = None,
+            **kwargs,
     ):
         """The AuthorizationRequest endpoint
 
@@ -986,7 +1090,7 @@ class Authorization(Endpoint):
         cinfo = _context.cdb[_cid]
         # logger.debug("client {}: {}".format(_cid, cinfo))
 
-        # this apply the default optionally deny_unknown_scopes policy
+        # this applies the default option deny_unknown_scopes policy
         check_unknown_scopes_policy(request, _cid, _context)
 
         if http_info is None:
@@ -996,7 +1100,11 @@ class Authorization(Endpoint):
         if _cookies:
             logger.debug("parse_cookie@process_request")
             _session_cookie_name = _context.cookie_handler.name["session"]
-            _my_cookies = _context.cookie_handler.parse_cookie(_session_cookie_name, _cookies)
+            try:
+                _my_cookies = _context.cookie_handler.parse_cookie(_session_cookie_name, _cookies)
+            except Exception as err:
+                logger.info(f"Parse cookie failed due to: {err}")
+                _my_cookies = {}
         else:
             _my_cookies = {}
 
