@@ -1,12 +1,6 @@
-import base64
-import hashlib
 import json
 import logging
 
-from cryptography.fernet import Fernet
-from cryptojwt import as_unicode
-from cryptojwt.utils import as_bytes
-
 from idpyoidc.util import importer
 
 from .exception import OidcEndpointError
@@ -201,7 +195,7 @@ def execute(spec, **kwargs):
 
 
 # def get_logout_id(endpoint_context, user_id, client_id):
-#     _item = SessionInfo()
+#     _item = NodeInfo()
 #     _item.user_id = user_id
 #     _item.client_id = client_id
 #
